import os

from devdeck_core.decks.deck_controller import DeckController
from slack_sdk import WebClient

from devdeck_slack.slack_away_control import SlackAwayControl
from devdeck_slack.slack_dnd_control import SlackDndControl
from devdeck_slack.slack_online_control import SlackOnlineControl
from devdeck_slack.slack_status_control import SlackStatusControl


class SlackDeck(DeckController):
    def __init__(self, key_no, **kwargs):
        self.actions = {
            'online': SlackOnlineControl,
            'away': SlackAwayControl,
            'status': SlackStatusControl,
            'dnd': SlackDndControl
        }
        super().__init__(key_no, **kwargs)

    def initialize(self):
        with self.deck_context() as context:
            with context.renderer() as r:
                r.image(os.path.join(os.path.join(os.path.dirname(__file__), "assets", 'slack.png'))).end()

    def deck_controls(self):
        api_client = WebClient(token=self.settings['api_key'])

        for action_setting in self.settings['actions']:
            action_control_class = self.actions[action_setting['action']]

            control_settings = dict(action_setting)
            del control_settings['action']
            del control_settings['key']
            self.register_control(action_setting['key'], action_control_class, api_client=api_client, **control_settings)

    def settings_schema(self):
        return {
            'api_key': {
                'type': 'string',
                'required': True,
            },
            'actions': {
                'type': 'list',
                'required': True,
                'schema': {
                    'type': 'dict',
                    'schema': {
                        'action': {
                            'type': 'string',
                            'required': True
                        },
                        'key': {
                            'type': 'integer',
                            'required': True
                        },
                        'text': {
                            'type': 'string',
                            'required': False
                        },
                        'emoji': {
                            'type': 'string',
                            'required': False
                        },
                        'duration': {
                            'type': 'integer',
                            'required': False
                        }
                    }
                }
            },
        }



from unittest import mock

from devdeck_core.mock_deck_context import mock_context, assert_rendered
from devdeck_core.renderer import Renderer

from devdeck_slack.slack_status_control import SlackStatusControl


class TestSlackStatusControl:
    @mock.patch('slack_sdk.WebClient')
    def test_initialize_sets_icon(self, api_client):
        control = SlackStatusControl(0, api_client, **{'emoji': ':sandwich:', 'text': 'On lunch'})
        with mock_context(control) as ctx:
            control.initialize()
            assert_rendered(ctx, Renderer().emoji('sandwich').end())

    @mock.patch('slack_sdk.WebClient')
    def test_pressed_sets_status(self, api_client):
        control = SlackStatusControl(0, api_client, **{'emoji': ':sandwich:', 'text': 'On lunch'})
        with mock_context(control) as ctx:
            control.pressed()
            api_client.users_profile_set.assert_called_with(profile={
                'status_text': 'On lunch',
                'status_emoji': ':sandwich:'
            })



from unittest import mock

from devdeck_core.mock_deck_context import mock_context, assert_rendered

from tests.testing_utils import TestingUtils

from devdeck_slack.slack_away_control import SlackAwayControl


class TestSlackAwayControl:
    @mock.patch('slack_sdk.WebClient')
    def test_initialize_sets_icon(self, api_client):
        control = SlackAwayControl(0, api_client, **{})
        with mock_context(control) as ctx:
            control.initialize()
            assert_rendered(ctx, TestingUtils.get_filename('../devdeck_slack/assets/away.png'))

    @mock.patch('slack_sdk.WebClient')
    def test_pressed_sets_presence_away(self, api_client):
        control = SlackAwayControl(0, api_client, **{})
        with mock_context(control) as ctx:
            control.pressed()
            api_client.users_setPresence.assert_called_with(presence='away')



from devdeck_core.controls.deck_control import DeckControl


class SlackDndControl(DeckControl):
    def __init__(self, key_no, api_client, **kwargs):
        self.api_client = api_client
        super().__init__(key_no, **kwargs)

    def initialize(self):
        with self.deck_context() as context:
            with context.renderer() as r:
                r.emoji('zzz')\
                    .end()\
                .badge_count(str(self.settings['duration']))\
                    .end()

    def pressed(self):
        self.api_client.dnd_setSnooze(num_minutes=self.settings['duration'])

    def settings_schema(self):
        return {
            'duration': {
                'type': 'integer',
                'required': True,
            }
        }


import os

from devdeck_core.controls.deck_control import DeckControl


class SlackAwayControl(DeckControl):
    def __init__(self, key_no, api_client, **kwargs):
        self.api_client = api_client
        super().__init__(key_no, **kwargs)

    def initialize(self):
        with self.deck_context() as context:
            with context.renderer() as r:
                r.image(os.path.join(os.path.dirname(__file__), "assets", 'away.png')).end()

    def pressed(self):
        self.api_client.users_setPresence(presence='away')


from devdeck_core.controls.deck_control import DeckControl


class SlackStatusControl(DeckControl):
    def __init__(self, key_no, api_client, **kwargs):
        self.api_client = api_client
        super().__init__(key_no, **kwargs)

    def initialize(self):
        with self.deck_context() as context:
            with context.renderer() as r:
                r.emoji(self.settings['emoji'])\
                    .end()

    def pressed(self):
        self.api_client.users_profile_set(profile={
            "status_text": self.settings['text'],
            "status_emoji": self.settings['emoji']
        })

    def settings_schema(self):
        return {
            'text': {
                'type': 'string',
                'required': True,
            },
            'emoji': {
                'type': 'string',
                'required': True,
            },
        }


from setuptools import setup, find_packages
import os
import subprocess


def get_version():
    process = subprocess.Popen(["git", "describe", "--always", "--tags"], stdout=subprocess.PIPE, stderr=None)
    last_tag = process.communicate()[0].decode('ascii').strip()
    if '-g' in last_tag:
        return last_tag.split('-g')[0].replace('-', '.')
    else:
        return last_tag


with open(os.path.abspath(os.path.join(os.path.dirname(__file__), 'requirements.txt'))) as f:
    install_reqs = f.read().splitlines()

setup(
    name='devdeck_slack',
    version=get_version(),
    description="Slack deck and controls for DevDeck.",
    long_description=open('README.md').read(),
    long_description_content_type='text/markdown',
    author='James Ridgway',
    url='https://github.com/jamesridgway/devdeck-slack',
    license='MIT',
    packages=find_packages(),
    include_package_data=True,
    install_requires=install_reqs
)



from unittest import mock

from devdeck_core.mock_deck_context import mock_context, assert_rendered
from devdeck_core.renderer import Renderer

from devdeck_slack.slack_dnd_control import SlackDndControl


class TestSlackDndControl:
    @mock.patch('slack_sdk.WebClient')
    def test_initialize_sets_icon(self, api_client):
        control = SlackDndControl(0, api_client, **{'duration': 15})
        with mock_context(control) as ctx:
            control.initialize()
            assert_rendered(ctx, Renderer().emoji('zzz').end().badge_count(15).end())

    @mock.patch('slack_sdk.WebClient')
    def test_pressed_sets_snooze(self, api_client):
        control = SlackDndControl(0, api_client, **{'duration': 15})
        with mock_context(control) as ctx:
            control.pressed()
            api_client.dnd_setSnooze.assert_called_with(num_minutes=15)



from unittest import mock

from devdeck_core.mock_deck_context import mock_context, assert_rendered

from devdeck_slack.slack_online_control import SlackOnlineControl
from tests.testing_utils import TestingUtils


class TestSlackOnlineControl:
    @mock.patch('slack_sdk.WebClient')
    def test_initialize_sets_icon(self, api_client):
        control = SlackOnlineControl(0, api_client, **{})
        with mock_context(control) as ctx:
            control.initialize()
            assert_rendered(ctx, TestingUtils.get_filename('../devdeck_slack/assets/online.png'))

    @mock.patch('slack_sdk.WebClient')
    def test_pressed_sets_presence_online(self, api_client):
        control = SlackOnlineControl(0, api_client, **{})
        with mock_context(control) as ctx:
            api_client.dnd_info.return_value = {'snooze_enabled': False}
            control.pressed()
            api_client.users_setPresence.assert_called_with(presence='auto')
            api_client.users_profile_set.assert_called_with(profile={"status_text": "", "status_emoji": ""})
            api_client.dnd_info.assert_called()
            api_client.dnd_endSnooze.assert_not_called()

    @mock.patch('slack_sdk.WebClient')
    def test_pressed_sets_presence_online_and_cancels_dnd(self, api_client):
        control = SlackOnlineControl(0, api_client, **{})
        with mock_context(control) as ctx:
            api_client.dnd_info.return_value = {'snooze_enabled': True}
            control.pressed()
            api_client.users_setPresence.assert_called_with(presence='auto')
            api_client.users_profile_set.assert_called_with(profile={"status_text": "", "status_emoji": ""})
            api_client.dnd_info.assert_called()
            api_client.dnd_endSnooze.assert_called()
